==> manaplus_1.3.7.21.orig.tar.xz: manaplus-1.3.7.21/src/textparticle.cpp <==
/*
 *  The ManaPlus Client
 *  Copyright (C) 2006-2009  The Mana World Development Team
 *  Copyright (C) 2009-2010  The Mana Developers
 *  Copyright (C) 2011-2013  The ManaPlus Developers
 *
 *  This file is part of The ManaPlus Client.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "textparticle.h"

#include "graphics.h"

#include "gui/theme.h"

#include <guichan/color.hpp>
#include <guichan/font.hpp>

#include "debug.h"

TextParticle::TextParticle(Map *const map, const std::string &text,
                           const gcn::Color *const color,
                           gcn::Font *const font, const bool outline) :
    Particle(map),
    mText(text),
    mTextFont(font),
    mColor(color),
    mOutline(outline),
    mTextWidth(mTextFont ? mTextFont->getWidth(mText) / 2 : 1)
{
}

bool TextParticle::draw(Graphics *const graphics,
                        const int offsetX, const int offsetY) const
{
    if (!mColor || !mTextFont)
        return false;

    BLOCK_START("TextParticle::draw")
    if (!isAlive())
    {
        BLOCK_END("TextParticle::draw")
        return false;
    }

    const int screenX = static_cast<int>(mPos.x) + offsetX;
    const int screenY = static_cast<int>(mPos.y) - static_cast<int>(mPos.z)
        + offsetY;

    float alpha = mAlpha * 255.0f;

    if (mFadeOut && mLifetimeLeft > -1 && mLifetimeLeft < mFadeOut)
    {
        alpha *= static_cast<float>(mLifetimeLeft)
                / static_cast<float>(mFadeOut);
    }

    if (mFadeIn && mLifetimePast < mFadeIn)
    {
        alpha *= static_cast<float>(mLifetimePast)
                / static_cast<float>(mFadeIn);
    }

    gcn::Color color = *mColor;
    color.a = static_cast<int>(alpha);

    graphics->setColor(color);
    if (mOutline)
    {
        graphics->setColor2(Theme::getThemeColor(
            Theme::OUTLINE, static_cast<int>(alpha)));
    }
    mTextFont->drawString(graphics, mText, screenX - mTextWidth, screenY);
    BLOCK_END("TextParticle::draw")
    return true;
}

==> manaplus-1.3.8.4.tar.xz: manaplus-1.3.8.4/src/textparticle.cpp <==
/*
 *  The ManaPlus Client
 *  Copyright (C) 2006-2009  The Mana World Development Team
 *  Copyright (C) 2009-2010  The Mana Developers
 *  Copyright (C) 2011-2013  The ManaPlus Developers
 *
 *  This file is part of The ManaPlus Client.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "textparticle.h"

#include "graphics.h"

#include "gui/theme.h"

#include <guichan/color.hpp>
#include <guichan/font.hpp>

#include "debug.h"

TextParticle::TextParticle(Map *const map, const std::string &text,
                           const gcn::Color *const color,
                           gcn::Font *const font, const bool outline) :
    Particle(map),
    mText(text),
    mTextFont(font),
    mColor(color),
    mTextWidth(mTextFont ? mTextFont->getWidth(mText) / 2 : 1),
    mOutline(outline)
{
}

bool TextParticle::draw(Graphics *const graphics,
                        const int offsetX, const int offsetY) const
{
    if (!mColor || !mTextFont)
        return false;

    BLOCK_START("TextParticle::draw")
    if (!isAlive())
    {
        BLOCK_END("TextParticle::draw")
        return false;
    }

    const int screenX = static_cast<int>(mPos.x) + offsetX;
    const int screenY = static_cast<int>(mPos.y) - static_cast<int>(mPos.z)
        + offsetY;

    float alpha = mAlpha * 255.0f;

    if (mFadeOut && mLifetimeLeft > -1 && mLifetimeLeft < mFadeOut)
    {
        alpha *= static_cast<float>(mLifetimeLeft)
                / static_cast<float>(mFadeOut);
    }

    if (mFadeIn && mLifetimePast < mFadeIn)
    {
        alpha *= static_cast<float>(mLifetimePast)
                / static_cast<float>(mFadeIn);
    }

    gcn::Color color = *mColor;
    color.a = static_cast<int>(alpha);

    graphics->setColor(color);
    if (mOutline)
    {
        graphics->setColor2(Theme::getThemeColor(
            Theme::OUTLINE, static_cast<int>(alpha)));
    }
    mTextFont->drawString(graphics, mText, screenX - mTextWidth, screenY);
    BLOCK_END("TextParticle::draw")
    return true;
}
